package info.aspirecn.cloud.yysj.controller;

import info.aspirecn.cloud.yysj.entity.CarQueryParam;
import info.aspirecn.cloud.yysj.entity.ResponseInfo;
import info.aspirecn.cloud.yysj.entity.Car;
import info.aspirecn.cloud.yysj.entity.CarInfo;
import info.aspirecn.cloud.yysj.service.CarSercice;
import info.aspirecn.cloud.yysj.utils.Constant;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 查询车辆列表、车辆详情，接口
 *
 * @author lixinxu
 */
@Api(tags = "车辆查询接口")
@ApiResponses({
        @ApiResponse(code = 200, message = "成功处理请求"),
        @ApiResponse(code = 400, message = "没有找到该用户信息"),
        @ApiResponse(code = 401, message = "没有权限访问该服务"),
        @ApiResponse(code = 403, message = "权限不足无法访问该服务"),
        @ApiResponse(code = 404, message = "未发现该微服务"),
        @ApiResponse(code = 500, message = "服务器内部错误")
})
@RestController
public class CarController {

    @Autowired
    CarSercice carSercice;

    @ApiOperation(value = "车辆列表", notes = "查询车辆信息列表")
    @PostMapping("/querycar")
    private ResponseInfo queryCarList(HttpServletResponse response,
                                      @ApiParam(value="用户ID")@RequestHeader()  String userId,
                                      @RequestBody CarQueryParam carQueryParam){

        carQueryParam.setUserId(userId);
        carQueryParam.setBeginTime(carQueryParam.getBeginTime());
        carQueryParam.setEndTime(carQueryParam.getEndTime());
        carQueryParam.setPageSize(carQueryParam.getPageSize());
        carQueryParam.setPageNum(carQueryParam.getPageNum());

        ResponseInfo responseInfo = new ResponseInfo();
        //map里有数据列表和查询总条数
        Map<String, Object> reMap = carSercice.queryCarList(carQueryParam);
        if (reMap == null){
            responseInfo.setMessage("数据深度不能大于1000条");
            return responseInfo;
        }

        List<Car> carList = (List<Car>)reMap.get(Constant.DATA_LIST);
        int total = (int) reMap.get(Constant.TOTAL);

        //页面信息, 放到响应头
        response.addHeader("pageSize", Integer.toString(carQueryParam.getPageSize()));
        response.addHeader("pageNum", Integer.toString(carQueryParam.getPageNum()));
        response.addHeader("pageTotal", Integer.toString(total%carQueryParam.getPageSize() > 0 ? total/carQueryParam.getPageSize()+1 : total/carQueryParam.getPageSize()));
        response.addHeader("total", Integer.toString(total));

        return new ResponseInfo().setData(carList);
    }

    @ApiOperation(value = "车辆详情", notes = "查询车辆详情")
    @GetMapping("/querycarinfo/{carId}")
    private ResponseInfo queryCarInfo(@PathVariable(required = true) String carId){
        CarInfo carInfo = carSercice.queryCarInfo(carId);
        return new ResponseInfo().setData(carInfo);
    }

}
