package info.aspirecn.cloud.yysj.utils;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.util.JSONPObject;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.rest.RestStatus;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class ElasticSearchUitls {

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 获取查询结果
     *
     * @param searchRequest 查询请求对象
     * @return 结果数据集合
     */
    public <T> Map<String, Object> getResult(SearchRequest searchRequest, Class<T> type) {
        // 设置返回集合
        Map<String, Object> resultMap = new HashMap<String, Object>();
        List<T> list = new ArrayList<>();
        //将数据放入map
        resultMap.put(Constant.DATA_LIST, list);
        resultMap.put(Constant.TOTAL, 0L);
        // 使用 RestHighLevelClient 客户端进行请求
        try {
            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            // 根据状态和数据条数验证是否返回了数据
            if (RestStatus.OK.equals(searchResponse.status()) && searchResponse.getHits().totalHits > 0) {
                //获取本页数据
                SearchHits hits = searchResponse.getHits();
                //将数据总条数放入map
                resultMap.put(Constant.TOTAL, hits.totalHits);
                for (SearchHit hit : hits) {
                    // 将 JSON 转换成对象
                    String hitStr = hit.getSourceAsString();
                    T t = objectMapper.readValue(hit.getSourceAsString(), type);

                    //如果有id字段, 存入id
                    try {
                        Field field = type.getDeclaredField("id");
                        field.setAccessible(true);
                        field.set(t, hit.getId());
                    } catch (Exception e) {
                    }

                    // 加入到对象集合
                    list.add(t);
                }
            }
        } catch (IOException e) {
            log.error("", e);
        }
        return resultMap;
    }

    public SearchHits getResultHits(SearchRequest searchRequest) {
        // 使用 RestHighLevelClient 客户端进行请求
        try {
            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            // 根据状态和数据条数验证是否返回了数据
            if (RestStatus.OK.equals(searchResponse.status()) && searchResponse.getHits().totalHits > 0) {
                //获取本页数据
                SearchHits hits = searchResponse.getHits();
                return hits;
            }
        } catch (IOException e) {
            log.error("", e);
        }
        return null;
    }

}
